@@ -1,7 +1,5 @@
 {-# LANGUAGE ExistentialQuantification, FlexibleInstances, GeneralizedNewtypeDeriving,
              MultiParamTypeClasses, TypeSynonymInstances, CPP, DeriveDataTypeable #-}
--- required for deriving Typeable
-{- # OPTIONS_GHC -fglasgow-exts #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -24,15 +22,17 @@ module XMonad.Core (
     XConf(..), XConfig(..), LayoutClass(..),
     Layout(..), readsLayout, Typeable, Message,
     SomeMessage(..), fromMessage, LayoutMessages(..),
+    StateExtension(..), ExtensionClass(..),
     runX, catchX, userCode, userCodeDef, io, catchIO, installSignalHandlers, uninstallSignalHandlers,
     withDisplay, withWindowSet, isRoot, runOnWorkspaces,
-    getAtom, spawn, spawnPID, getXMonadDir, recompile, trace, whenJust, whenX,
+    getAtom, spawn, spawnPID, xfork, getXMonadDir, recompile, trace, whenJust, whenX,
     atom_WM_STATE, atom_WM_PROTOCOLS, atom_WM_DELETE_WINDOW, ManageHook, Query(..), runQuery
   ) where
 
 import XMonad.StackSet hiding (modify)
 
 import Prelude hiding ( catch )
+import Codec.Binary.UTF8.String (encodeString)
 import Control.Exception.Extensible (catch, fromException, try, bracket, throw, finally, SomeException(..))
 import Control.Applicative
 import Control.Monad.State
@@ -51,19 +51,25 @@ import Graphics.X11.Xlib
 import Graphics.X11.Xlib.Extras (Event)
 import Data.Typeable
 import Data.List ((\\))
-import Data.Maybe (isJust)
+import Data.Maybe (isJust,fromMaybe)
 import Data.Monoid
-import Data.Maybe (fromMaybe)
 
 import qualified Data.Map as M
 import qualified Data.Set as S
 
 -- | XState, the (mutable) window manager state.
 data XState = XState
-    { windowset    :: !WindowSet           -- ^ workspace list
-    , mapped       :: !(S.Set Window)      -- ^ the Set of mapped windows
-    , waitingUnmap :: !(M.Map Window Int)  -- ^ the number of expected UnmapEvents
-    , dragging     :: !(Maybe (Position -> Position -> X (), X ())) }
+    { windowset        :: !WindowSet                     -- ^ workspace list
+    , mapped           :: !(S.Set Window)                -- ^ the Set of mapped windows
+    , waitingUnmap     :: !(M.Map Window Int)            -- ^ the number of expected UnmapEvents
+    , dragging         :: !(Maybe (Position -> Position -> X (), X ()))
+    , numberlockMask   :: !KeyMask                       -- ^ The numlock modifier
+    , extensibleState  :: !(M.Map String (Either String StateExtension))
+    -- ^ stores custom state information.
+    --
+    -- The module "XMonad.Utils.ExtensibleState" in xmonad-contrib
+    -- provides additional information and a simple interface for using this.
+    }
 
 -- | XConf, the (read-only) window manager configuration.
 data XConf = XConf
@@ -93,7 +99,6 @@ data XConfig l = XConfig
                                                  -- should also be run afterwards. mappend should be used for combining
                                                  -- event hooks in most cases.
     , workspaces         :: ![String]            -- ^ The list of workspaces' names
-    , numlockMask        :: !KeyMask             -- ^ The numlock modifier
     , modMask            :: !KeyMask             -- ^ the mod modifier
     , keys               :: !(XConfig Layout -> M.Map (ButtonMask,KeySym) (X ()))
                                                  -- ^ The key binding: a map from key presses and actions
@@ -129,9 +134,7 @@ data ScreenDetail   = SD { screenRect :: !Rectangle } deriving (Eq,Show, Read)
 -- instantiated on 'XConf' and 'XState' automatically.
 --
 newtype X a = X (ReaderT XConf (StateT XState IO) a)
-#ifndef __HADDOCK__
     deriving (Functor, Monad, MonadIO, MonadState XState, MonadReader XConf, Typeable)
-#endif
 
 instance Applicative X where
   pure = return
@@ -143,9 +146,7 @@ instance (Monoid a) => Monoid (X a) where
 
 type ManageHook = Query (Endo WindowSet)
 newtype Query a = Query (ReaderT Window X a)
-#ifndef __HADDOCK__
     deriving (Functor, Monad, MonadReader Window, MonadIO)
-#endif
 
 runQuery :: Query a -> Window -> X a
 runQuery (Query m) w = runReaderT m w
@@ -343,6 +344,33 @@ data LayoutMessages = Hide              -- ^ sent when a layout becomes non-visi
 
 instance Message LayoutMessages
 
+-- ---------------------------------------------------------------------
+-- Extensible state
+--
+
+-- | Every module must make the data it wants to store
+-- an instance of this class.
+--
+-- Minimal complete definition: initialValue
+class Typeable a => ExtensionClass a where
+    -- | Defines an initial value for the state extension
+    initialValue :: a
+    -- | Specifies whether the state extension should be
+    -- persistent. Setting this method to 'PersistentExtension'
+    -- will make the stored data survive restarts, but
+    -- requires a to be an instance of Read and Show.
+    --
+    -- It defaults to 'StateExtension', i.e. no persistence.
+    extensionType :: a -> StateExtension
+    extensionType = StateExtension
+
+-- | Existential type to store a state extension.
+data StateExtension =
+    forall a. ExtensionClass a => StateExtension a
+    -- ^ Non-persistent state extension
+  | forall a. (Read a, Show a, ExtensionClass a) => PersistentExtension a
+    -- ^ Persistent extension
+
 -- ---------------------------------------------------------------------
 -- | General utilities
 --
@@ -356,16 +384,22 @@ catchIO :: MonadIO m => IO () -> m ()
 catchIO f = io (f `catch` \(SomeException e) -> hPrint stderr e >> hFlush stderr)
 
 -- | spawn. Launch an external application. Specifically, it double-forks and
--- runs the 'String' you pass as a command to /bin/sh.
+-- runs the 'String' you pass as a command to \/bin\/sh.
+--
+-- Note this function assumes your locale uses utf8.
 spawn :: MonadIO m => String -> m ()
 spawn x = spawnPID x >> return ()
 
 -- | Like 'spawn', but returns the 'ProcessID' of the launched application
 spawnPID :: MonadIO m => String -> m ProcessID
-spawnPID x = io . forkProcess . finally nullStdin $ do
+spawnPID x = xfork $ executeFile "/bin/sh" False ["-c", encodeString x] Nothing
+
+-- | A replacement for 'forkProcess' which resets default signal handlers.
+xfork :: MonadIO m => IO () -> m ProcessID
+xfork x = io . forkProcess . finally nullStdin $ do
                 uninstallSignalHandlers
                 createSession
-                executeFile "/bin/sh" False ["-c", x] Nothing
+                x
  where
     nullStdin = do
         fd <- openFd "/dev/null" ReadOnly Nothing defaultFileFlags
@@ -393,9 +427,11 @@ getXMonadDir = io $ getAppUserDataDirectory "xmonad"
 --
 --      * the xmonad executable does not exist
 --
---      * the xmonad executable is older than xmonad.hs
+--      * the xmonad executable is older than xmonad.hs or any file in
+--        ~\/.xmonad\/lib
 --
--- The -i flag is used to restrict recompilation to the xmonad.hs file only.
+-- The -i flag is used to restrict recompilation to the xmonad.hs file only,
+-- and any files in the ~\/.xmonad\/lib directory.
 --
 -- Compilation errors (if any) are logged to ~\/.xmonad\/xmonad.errors.  If
 -- GHC indicates failure with a non-zero exit code, an xmessage displaying
@@ -419,7 +455,7 @@ recompile force = io $ do
       then do
         -- temporarily disable SIGCHLD ignoring:
         uninstallSignalHandlers
-        status <- bracket (openFile err WriteMode) hClose $ \h -> do
+        status <- bracket (openFile err WriteMode) hClose $ \h ->
             waitForProcess =<< runProcess "ghc" ["--make", "xmonad.hs", "-i", "-ilib", "-fforce-recomp", "-v0", "-o",binn] (Just dir)
                                     Nothing Nothing Nothing (Just h)
 
@@ -431,7 +467,8 @@ recompile force = io $ do
             ghcErr <- readFile err
             let msg = unlines $
                     ["Error detected while loading xmonad configuration file: " ++ src]
-                    ++ lines ghcErr ++ ["","Please check the file for errors."]
+                    ++ lines (if null ghcErr then show status else ghcErr)
+                    ++ ["","Please check the file for errors."]
             -- nb, the ordering of printing, then forking, is crucial due to
             -- lazy evaluation
             hPutStrLn stderr msg
